﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;
using Kendo.Mvc.Extensions;
using Kendo.Mvc.UI;
using Maples.eServices.UI.Authorisation;
using Maples.eServices.UI.Models.StructureChart;

namespace Maples.eServices.UI.Controllers
{
	/// <summary>
	/// The structure chart controller
	/// </summary>
	[AllowCompany]
	public class StructureChartController : BaseController
	{
		#region Public Methods

		/// <summary>
		/// Gets the structure Chart for a company
		/// </summary>
		/// <returns>The structure chart view</returns>
		[HttpGet]
		public ActionResult StructureChart()
		{
			var model = new StructureChartSelectionModel
			{
				CreateFullChart = true,
				ParentDepth = null,
				ChildDepth = null,
        Data = GetStructureChart()
			};
			
			return View(model);
		}

		/// <summary>
		/// Gets the structure chart for the current entity
		/// </summary>
		/// <returns>A result model</returns>
    private StructureChartResultsModel GetStructureChart()
		{
			var model = new StructureChartResultsModel { BaseCompany = CurrentCompany};
		  GetTestData(model);
		  return model;
		}

    /// <summary>
    /// Gets the shapes.
    /// </summary>
    /// <param name="request">The request</param>
    /// <returns>The shapes</returns>
    [HttpPost]
    public ActionResult GetShapes([DataSourceRequest] DataSourceRequest request)
    {
      return Json(GetStructureChart().Entities.ToDataSourceResult(request), JsonRequestBehavior.AllowGet);
    }

    /// <summary>
    /// Gets the connections.
    /// </summary>
    /// <param name="request">The request</param>
    /// <returns>The connections</returns>
    [HttpPost]
    public ActionResult GetConnections([DataSourceRequest] DataSourceRequest request)
	  {
      return Json(GetStructureChart().Links.ToDataSourceResult(request), JsonRequestBehavior.AllowGet);
	  }

		/// <summary>
		/// Saves the export of the structure chart
		/// </summary>
		/// <param name="contentType">Type of the export content</param>
		/// <param name="base64">The base64 string</param>
		/// <param name="fileName">Name of the file</param>
		/// <returns>The saved file of the export</returns>
		[HttpPost]
		public ActionResult ExportSave(string contentType, string base64, string fileName)
		{
			var fileContents = Convert.FromBase64String(base64);
			return File(fileContents, contentType, fileName);
		}

		#endregion

		#region Test Data (DELETE THIS)

		/// <summary>
		/// Gets the test data for the POC for structure charts
		/// </summary>
    /// <param name="model">The model</param>
		private static void GetTestData(StructureChartResultsModel model)
		{
			model.Entities = new List<StructureChartEntityModel>();
      model.Links = new List<StructureChartLinkModel>();

			// Create some demo entities
			model.Entities.Add(new StructureChartEntityModel { Id = 1, EntityName = "Test Company 1", EntityTypeEnum = EntityType.Company,  Jurisdiction = "Cayman" });
      model.Entities.Add(new StructureChartEntityModel { Id = 2, EntityName = "Test Company 2", EntityTypeEnum = EntityType.Company, Jurisdiction = "Cayman" });
      model.Entities.Add(new StructureChartEntityModel { Id = 3, EntityName = "Test Partnership 3", EntityTypeEnum = EntityType.Partnership, Jurisdiction = "Cayman" });
      model.Entities.Add(new StructureChartEntityModel { Id = 4, EntityName = "Test Trust 4", EntityTypeEnum = EntityType.Trust, Jurisdiction = "Delaware" });
      model.Entities.Add(new StructureChartEntityModel { Id = 5, EntityName = "Test Trust 5", EntityTypeEnum = EntityType.Trust, Jurisdiction = "Delaware" });
      model.Entities.Add(new StructureChartEntityModel { Id = 6, EntityName = "Test Company 6", EntityTypeEnum = EntityType.Company, Jurisdiction = "Cayman" });
      model.Entities.Add(new StructureChartEntityModel { Id = 7, EntityName = "Test Company 7", EntityTypeEnum = EntityType.Company, Jurisdiction = "Cayman" });
      model.Entities.Add(new StructureChartEntityModel { Id = 8, EntityName = "Test Company 8", EntityTypeEnum = EntityType.Company, Jurisdiction = "Cayman" });
      model.Entities.Add(new StructureChartEntityModel { Id = 9, EntityName = "Test Partnership 9", EntityTypeEnum = EntityType.Partnership, Jurisdiction = "Cayman" });
      model.Entities.Add(new StructureChartEntityModel { Id = 10, EntityName = "Test Trust 10", EntityTypeEnum = EntityType.Trust, Jurisdiction = "Delaware" });
      model.Entities.Add(new StructureChartEntityModel { Id = 11, EntityName = "Test Trust 11", EntityTypeEnum = EntityType.Trust, Jurisdiction = "Delaware" });
      model.Entities.Add(new StructureChartEntityModel { Id = 12, EntityName = "Test Company 12", EntityTypeEnum = EntityType.Company, Jurisdiction = "Cayman" });
      model.Entities.Add(new StructureChartEntityModel { Id = 13, EntityName = "Test Company 13", EntityTypeEnum = EntityType.Company, Jurisdiction = "Cayman" });
      model.Entities.Add(new StructureChartEntityModel { Id = 14, EntityName = "Test Company 14", EntityTypeEnum = EntityType.Company, Jurisdiction = "Cayman" });
      model.Entities.Add(new StructureChartEntityModel { Id = 15, EntityName = "Test Partnership 15", EntityTypeEnum = EntityType.Partnership, Jurisdiction = "Cayman" });
      model.Entities.Add(new StructureChartEntityModel { Id = 16, EntityName = "Test Trust 16", EntityTypeEnum = EntityType.Trust, Jurisdiction = "Delaware" });
      model.Entities.Add(new StructureChartEntityModel { Id = 17, EntityName = "Test Trust 17", EntityTypeEnum = EntityType.Trust, Jurisdiction = "Delaware" });
      model.Entities.Add(new StructureChartEntityModel { Id = 18, EntityName = "Test Company 18", EntityTypeEnum = EntityType.Company, Jurisdiction = "Cayman" });

			// Create the relationships between the entities
      model.Links.Add(new StructureChartLinkModel() { Id = 1, FromId = 1, ToId = 2, Description = "Test Description"});
      model.Links.Add(new StructureChartLinkModel() { Id = 16, FromId = 18, ToId = 15 });
      model.Links.Add(new StructureChartLinkModel() { Id = 15, FromId = 17, ToId = 16 });
      model.Links.Add(new StructureChartLinkModel() { Id = 2, FromId = 2, ToId = 5 });
      model.Links.Add(new StructureChartLinkModel() { Id = 3, FromId = 3, ToId = 5 });
      model.Links.Add(new StructureChartLinkModel() { Id = 4, FromId = 4, ToId = 5 });
      model.Links.Add(new StructureChartLinkModel() { Id = 4, FromId = 5, ToId = 6 });
      model.Links.Add(new StructureChartLinkModel() { Id = 5, FromId = 6, ToId = 7 });
      model.Links.Add(new StructureChartLinkModel() { Id = 6, FromId = 7, ToId = 8 });
      model.Links.Add(new StructureChartLinkModel() { Id = 7, FromId = 8, ToId = 9 });
      model.Links.Add(new StructureChartLinkModel() { Id = 8, FromId = 8, ToId = 10 });
      model.Links.Add(new StructureChartLinkModel() { Id = 9, FromId = 9, ToId = 11 });
      model.Links.Add(new StructureChartLinkModel() { Id = 10, FromId = 11, ToId = 12 });
      model.Links.Add(new StructureChartLinkModel() { Id = 11, FromId = 5, ToId = 13 });
      model.Links.Add(new StructureChartLinkModel() { Id = 12, FromId = 14, ToId = 13 });
      model.Links.Add(new StructureChartLinkModel() { Id = 13, FromId = 15, ToId = 14 });
      model.Links.Add(new StructureChartLinkModel() { Id = 14, FromId = 16, ToId = 14 });
      
      
		}

		#endregion
	}
}